//==============================================================================================================
// USER MODIFIABLE PARAMETERS
//==============================================================================================================

//--------------------------------------------------------------------------------------------------------------
// The ifndef is used to exclude these when this header is used to build the library itself
//--------------------------------------------------------------------------------------------------------------
#ifndef LIBBUILD
//--------------------------------------------------------------------------------------------------------------
// System Timing Specifications - depends on user's hardware configuration and CONFIG bit settings
//--------------------------------------------------------------------------------------------------------------
#define Fosc                8000000     // Oscillator frequency in Hz (raw - before any divides or PLL)
#define FRCDIV              1           // 1:x Ratio specified in CLKDIVbits.RCDIV

//--------------------------------------------------------------------------------------------------------------
// COUNT and RANGE control the sensitivity of the debounce routine.  Whenever a switch press is detected, a
// timer is initialized to the value COUNT=16383 and the switch will be checked repeatedly in a loop.
// * Each time the switch is detected as "pressed", COUNT is incremented.
// * Each time the switch is detected as "not pressed", COUNT is decremented.
// * If COUNT reaches the value (COUNT + RANGE), the switch is declared "pressed" and the loop ends.
// * If COUNT reaches the value (COUNT - RANGE), the switch is declared "not pressed" and the loop ends.
//--------------------------------------------------------------------------------------------------------------
#define RANGE 80u                        // Max deviation from "midpoint" for valid (+) or false (-) counts

//==============================================================================================================
// CALCULATED CONSTANTS
//==============================================================================================================

//--------------------------------------------------------------------------------------------------------------
// Instruction cycle rate calculated from user modifiable parameters above
//--------------------------------------------------------------------------------------------------------------
#define Fcy                 ((Fosc/2) / FRCDIV)

//--------------------------------------------------------------------------------------------------------------
// LCD Library constants calculated from user modifiable parameters above
//--------------------------------------------------------------------------------------------------------------
const unsigned int LCD_F_INSTR = (unsigned)(0.00005 * Fcy);     // 50us * Fcy
const unsigned int LCD_S_INSTR = (unsigned)(0.002 * Fcy);       // 2ms * Fcy
const unsigned int LCD_STARTUP = (unsigned)(0.004 * Fcy);       // 4ms * Fcy

const unsigned int COUNT = 16383u;
const unsigned int MAX_COUNT = (unsigned)(16383u + RANGE);                 // When debounce count reaches this value, the switch is pressed
const unsigned int MIN_COUNT = (unsigned)(16383u - RANGE);                 // When debounce count reaches this value, the switch is NOT pressed
#endif

//==============================================================================================================
// LCD COMMAND CONSTANTS
//==============================================================================================================

//--------------------------------------------------------------------------------------------------------------
// LCD Command values to avoid magic numbers
//--------------------------------------------------------------------------------------------------------------
#define LCD_CLEAR           0x01
#define LCD_HOME            0x02
#define LCD_CURSOR_BACK     0x10
#define LCD_CURSOR_FWD      0x14
#define LCD_PAN_LEFT        0x18
#define LCD_PAN_RIGHT       0x1C

//--------------------------------------------------------------------------------------------------------------
// Labels to use when specifying the radix parameter of the lcdPutInt() and lcdPutUInt() functions
//--------------------------------------------------------------------------------------------------------------
#define BIN                 2
#define OCT                 8
#define DEC                 10
#define HEX                 16

//--------------------------------------------------------------------------------------------------------------
// Labels to make references to the Explorer 16's switches more readable
//--------------------------------------------------------------------------------------------------------------
#define Switch_S3 PORTDbits.RD6
#define Switch_S6 PORTDbits.RD7
#define Switch_S5 PORTAbits.RA7
#define Switch_S4 PORTDbits.RD13

//==============================================================================================================
// LCD MACRO COMMANDS (specialized versions of generic commands)
//==============================================================================================================
#define lcdHome()       lcdPutSCmd(LCD_HOME)
#define lcdCurFwd()     lcdPutSCmd(LCD_CURSOR_FWD)
#define lcdCurBack()    lcdPutSCmd(LCD_CURSOR_BACK)
#define lcdPanLeft()    lcdPutSCmd(LCD_PAN_LEFT)
#define lcdPanRight()   lcdPutSCmd(LCD_PAN_RIGHT)
#define lcdClearRow(thisRow)    lcdPutCur(thisRow, 0); lcdPutStr("                "); lcdPutCur(thisRow, 0)

//==============================================================================================================
// LCD CONTROL FUNCTION PROTOTYPES
//==============================================================================================================
void lcdInit(void);
void lcdPutCmd(char cmd);
void lcdPutSCmd(char cmd);  //Used only by macros above
void lcdPutChar(char ch);
void lcdPutStr(char *str);
void lcdPutCur(char row, char col);
void lcdPutInt(long num, char radix);
void lcdPutUInt(unsigned long num, char radix);
void lcdPutFloat(float fnum, char precision);
void lcdClear(void);
unsigned SwitchPressed(volatile unsigned int *sw, unsigned bit);
unsigned SwitchReleased(volatile unsigned int *sw, unsigned bit);
